/*
 * A simple producer-consumer multithread application.
 */

#include <stdio.h>

#include <yarp/os/all.h>
#include <myheader.h>

using namespace yarp::os;
using namespace std;

/*
 * the shared buffer between the producer and the consumer.
 */
class MyBuffer
{
protected:
	int shared_buffer[N];
	int write_ptr;
	int read_ptr;

public:
	MyBuffer() { write_ptr = read_ptr = 0; }

	void inert_item (int item_value)
	{
		shared_buffer[write_ptr] = item_value;
		write_ptr = (write_ptr + 1) % N;
	}

	int remove_item (void)
	{
		int ret = shared_buffer[read_ptr];
		read_ptr = (read_ptr + 1) % N;
		return ret;
	}
};

MyBuffer buffer;		        // the instance of the buffer.

Semaphore mutex(1);			// mutual exclusion.
Semaphore empty(N);			// how many slots are empty.
Semaphore full(0);			// how many slots are full.

/*
 *
 */
class Producer : public Thread 
{
public:
	virtual void run() 
	{
		int count = 0;
		while (!isStopping()) 
		{
			empty.wait ();
			mutex.wait ();
			printf ("Producer: adding %d\n", count);
			buffer.inert_item (count);
			mutex.post ();
			full.post ();
			count++;
			Time::delay(2);
		}
	}
};

/*
 *
 */
class Consumer : public Thread 
{
public:
	virtual void run() 
	{
		int value;
		while (!isStopping()) 
		{
			full.wait ();
			mutex.wait ();
			value = buffer.remove_item ();
			printf ("Consumer: read %d\n", value);
			mutex.post ();
			empty.post ();
			Time::delay(1);
		}
	}
};


/*
 * our simple main, type any char to stop threads and exit.
 */
int main (int argc, char *argv[])
{
	Network yarp;
	/// 
	Producer p;
	Consumer c;

	printf ("Main: starting producer\n");
	p.start ();
	Time::delay (1);
	
	printf ("Main: starting consumer\n");
	c.start ();

	printf ("Type a key and ret to quit\n");
	char d;
	scanf ("%c", &d);

	c.stop ();
	p.stop ();

	return 0;
}

